@@ -12,18 +12,73 @@ BASE_LOG_FILENAME="nvidia-bug-report.log"
 GZIP_CMD=`which gzip 2> /dev/null | head -n 1`
 if [ $? -eq 0 -a "$GZIP_CMD" ]; then
     GZIP_CMD="gzip -c"
-    LOG_FILENAME="$BASE_LOG_FILENAME.gz"
-    OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old.gz"
 else
     GZIP_CMD="cat"
-    LOG_FILENAME=$BASE_LOG_FILENAME
-    OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old"
 fi
 
+set_filename() {
+    if [ "$GZIP_CMD" = "gzip -c" ]; then
+        LOG_FILENAME="$BASE_LOG_FILENAME.gz"
+        OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old.gz"
+    else
+        LOG_FILENAME=$BASE_LOG_FILENAME
+        OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old"
+    fi
+}
 
-NVIDIA_BUG_REPORT_CHANGE='$Change: 12718902 $'
+usage_bug_report_message() {
+    echo "Please include the '$LOG_FILENAME' log file when reporting"
+    echo "your bug to freebsd-gfx-bugs@nvidia.com."
+}
+
+usage() {
+    echo ""
+    echo "$(basename $0): NVIDIA FreeBSD Graphics Driver bug reporting shell script."
+    echo ""
+    usage_bug_report_message
+    echo ""
+    echo "$(basename $0) [ -h | --help ] [ -o | --output-file <file> ]"
+    echo "    -h / --help"
+    echo "        Print this help output and exit."
+    echo "    --output-file <file>"
+    echo "        Write output to <file>. If gzip is available, the output file"
+    echo "        will be automatically compressed, and \".gz\" will be appended"
+    echo "        to the filename. Default: write to nvidia-bug-report.log(.gz)."
+    echo ""
+}
+
+NVIDIA_BUG_REPORT_CHANGE='$Change: 14225148 $'
 NVIDIA_BUG_REPORT_VERSION=`echo "$NVIDIA_BUG_REPORT_CHANGE" | tr -c -d "[:digit:]"`
 
+# Set the default filename so that it won't be empty in the usage message
+set_filename
+
+# Parse arguments: Optionally set output file or print help
+BUG_REPORT_SAFE_MODE=0
+SAVED_FLAGS=$@
+while [ "$1" != "" ]; do
+    case $1 in
+        -o | --output-file )    if [ -z $2 ]; then
+                                    usage
+                                    exit 1
+                                elif [ "$(echo "$2" | cut -c 1)" = "-" ]; then
+                                    echo "Warning: Questionable filename"\
+                                         "\"$2\": possible missing argument?"
+                                fi
+                                BASE_LOG_FILENAME="$2"
+                                # override the default filename
+                                set_filename
+                                shift
+                                ;;
+        -h | --help )           usage
+                                exit
+                                ;;
+        * )                     usage
+                                exit 1
+    esac
+    shift
+done
+
 #
 # echo_metadata() - echo metadata of specified file
 #
@@ -151,6 +206,16 @@ fi
 
 # print a start message to stdout
 
+echo ""
+echo "$(basename $0) will now collect information about your"
+echo "system and create the file '$LOG_FILENAME' in the current"
+echo "directory.  It may take several seconds to run.  In some"
+echo "cases, it may hang trying to capture data generated dynamically"
+echo "by the FreeBSD kernel and/or the NVIDIA kernel module.  While"
+echo "the bug report log file will be incomplete if this happens, it"
+echo "may still contain enough data to diagnose your problem." 
+echo ""
+usage_bug_report_message
 echo ""
 echo -n "Running $(basename $0)...";
 
