@@ -21,8 +21,11 @@
 
 #include <sys/param.h>
 
-#if __FreeBSD_version >= 1000000
-#error This driver does not support FreeBSD 10.x/-CURRENT!
+#if __FreeBSD_version >= 1100000
+#error This driver does not support FreeBSD 11.x/-CURRENT!
+#endif
+#if __FreeBSD_version >= 1000000 && __FreeBSD_version < 1000510
+#error This driver requires FreeBSD 10.0 or later!
 #endif
 #if __FreeBSD_version >= 900000 && __FreeBSD_version < 900044
 #error This driver requires FreeBSD 9.0 or later!
@@ -62,6 +65,7 @@
 #include <sys/mman.h>
 #include <sys/file.h>
 #include <sys/poll.h>
+#include <sys/rwlock.h>
 
 #include <sys/syscall.h>
 #include <sys/bus.h>
@@ -109,8 +113,11 @@
 
 #if __FreeBSD_version >= 900041
 #include <sys/capability.h>
+#if __FreeBSD_version < 1000053
+#define cap_rights_init(rights, cap) (cap)
+#endif
 #else
-#define fget(td, fd, cap, fp) fget(td, fd, fp)
+#define fget(td, fd, rights, fp) fget(td, fd, fp)
 #endif
 
 #define __NV_ITHREAD() (curthread->td_pflags & TDP_ITHREAD)
@@ -267,6 +274,10 @@ extern nv_state_t nvidia_ctl_state;
 #define PCIM_CMD_INTXDIS   0x400
 #endif
 
+#if !defined(VM_MEMATTR_WEAK_UNCACHEABLE)
+#define VM_MEMATTR_WEAK_UNCACHEABLE VM_MEMATTR_UNCACHED
+#endif
+
 #if !defined(PAT_UNCACHEABLE)
 #define PAT_UNCACHEABLE         0x00
 #endif
@@ -373,12 +384,27 @@ NvS32  nv_free_agp_pages     (nv_state_t *, NvU32, void *, NvU32 *);
         (obj) = NULL;                   \
     }
 
+#if __FreeBSD_version < 1000030
+#define VM_OBJECT_WLOCK(object) VM_OBJECT_LOCK(object)
+#define VM_OBJECT_WUNLOCK(object) VM_OBJECT_UNLOCK(object)
+#endif
+
+#if __FreeBSD_version < 1000055
+#define vm_map_find(map, object, offset, addr, length, \
+        max_addr, find_space, prot, max, cow) \
+  vm_map_find(map, object, offset, addr, length, find_space, prot, max, cow)
+#endif
+
+#if __FreeBSD_version < 1000042
+#define kmem_arena kernel_map
+#endif
+
 #if __FreeBSD_version >= 800000
-#define NV_VM_PAGER_ALLOCATE(tp,hd,sz,pt,of,cr) \
-  vm_pager_allocate(tp, hd, sz, pt, of, cr)
+#define NV_VM_PAGER_ALLOCATE(type, handle, size, prot, off, cred) \
+  vm_pager_allocate(type, handle, size, prot, off, cred)
 #else
-#define NV_VM_PAGER_ALLOCATE(tp,hd,sz,pt,of,cr) \
-  vm_pager_allocate(tp, hd, sz, pt, of)
+#define NV_VM_PAGER_ALLOCATE(type, handle, size, prot, off, cred) \
+  vm_pager_allocate(type, handle, size, prot, off)
 #endif
 
 #if __FreeBSD_version >= 800000
@@ -422,7 +448,7 @@ void   nvidia_linux_exit     (void);
 void   nvidia_update_registry (char *);
 
 /* ioctl helpers */
-int    nvidia_get_card_info  (struct nv_ioctl_card_info *);
+int    nvidia_get_card_info  (void *, int);
 int    nvidia_handle_ioctl   (nv_state_t *, struct nvidia_filep *, u_long, caddr_t);
 
 /* device helpers */
